import collections.abc
import inspect
import re
import types
import typing


def short(sig: inspect.Signature) -> str:
    out = str(sig)
    out = out.replace("tree_sitter_type_provider.node_types.", "")
    out = re.sub(r"ForwardRef\('(\w+)'\)", r"\1", out)
    out = out.replace("typing.", "")
    out = out.replace("NoneType", "None")
    out = out.replace("abc.", "")
    out = out.replace("~Result", "Result")
    return out


def function_signatures(object: object) -> collections.abc.Iterator[str]:
    for name, fun in inspect.getmembers(object, inspect.isfunction):
        assert isinstance(fun, types.FunctionType)
        if not name.startswith("_"):
            try:
                funsig = inspect.signature(fun)
                yield f"def {name}{short(funsig)}:"
                yield f"  pass"
            except ValueError:
                pass


def class_signatures(object: object) -> collections.abc.Iterator[str]:
    for name, cls in inspect.getmembers(object, inspect.isclass):
        if not name.startswith("_"):
            try:
                parent = cls.__mro__[1].__name__
                init = short(inspect.signature(cls))
                yield f"class {name}({parent}):"
                yield f"  def __init__{init}:"
                yield f"    pass"
            except ValueError:
                pass


def node_dict_simplify(node_dict: dict[str, typing.Any]) -> None:
    if len(node_dict) > 4:
        del node_dict["text"]

    del node_dict["start_position"]
    del node_dict["end_position"]

    for key in node_dict.keys():
        if isinstance(node_dict[key], dict):
            node_dict_simplify(node_dict[key])
        if isinstance(node_dict[key], list):
            for val in node_dict[key]:
                if isinstance(val, dict):
                    node_dict_simplify(val)



import pytest

from . import class_signatures, function_signatures


@pytest.mark.golden_test("data/api.yml")
def test_talon_api(golden):
    assert golden["input"] is None

    import tree_sitter_talon

    globals().update(tree_sitter_talon.__dict__)

    output: list[str] = []
    output.extend(function_signatures(tree_sitter_talon.__class__))
    output.extend(function_signatures(tree_sitter_talon))
    output.extend(class_signatures(tree_sitter_talon.__class__))
    output.extend(class_signatures(tree_sitter_talon))

    assert "\n".join(output) == golden.out["output"]



import collections.abc
import json
import os
import pathlib
import typing

import pkg_resources  # type: ignore
import tree_sitter  # type: ignore
import tree_sitter_type_provider

from .binding import _tree_sitter_talon_id


class TalonLanguage(tree_sitter.Language):
    def __init__(self):
        self.language_id = _tree_sitter_talon_id()


class TreeSitterTalon(tree_sitter_type_provider.TreeSitterTypeProvider):

    __version__: str = "3.0.5"

    _library_path: typing.Optional[str] = None

    _language: typing.Optional[tree_sitter.Language] = None

    _parser: typing.Optional[tree_sitter.Parser] = None

    @property
    def _node_types(
        self,
    ) -> collections.abc.Sequence[tree_sitter_type_provider.NodeType]:
        node_types_path = pathlib.Path(
            pkg_resources.resource_filename(
                "tree_sitter_talon",
                os.path.join("data", "tree-sitter-talon", "src", "node-types.json"),
            )
        )
        return tree_sitter_type_provider.NodeType.schema().loads(  # type: ignore
            node_types_path.read_text(), many=True
        )

    @property
    def __grammar_version__(self) -> str:
        package_json_path = pathlib.Path(
            pkg_resources.resource_filename(
                "tree_sitter_talon",
                os.path.join("data", "tree-sitter-talon", "package.json"),
            )
        )
        package_json = json.loads(package_json_path.read_text())
        return package_json["version"]

    @property
    def language(self) -> tree_sitter.Language:
        if not self._language:
            self._language = TalonLanguage()
        return self._language

    @property
    def parser(self) -> tree_sitter.Parser:
        if not self._parser:
            self._parser = tree_sitter.Parser()
            self._parser.set_language(self.language)
        return self._parser

    def __init__(self):
        # Conversion from tree-sitter names to Python names
        def as_class_name(node_type_name: str) -> str:
            buffer = ["Talon"]
            for part in node_type_name.split("_"):
                buffer.append(part.capitalize())
            return "".join(buffer)

        # Initialize module
        super().__init__(
            "tree_sitter_talon",
            self._node_types,
            as_class_name=as_class_name,
            extra=["comment"],
        )
        self.NodeTypeError = tree_sitter_type_provider.NodeTypeError
        self.NodeTypeName = tree_sitter_type_provider.NodeTypeName
        self.Node = tree_sitter_type_provider.Node
        self.Leaf = tree_sitter_type_provider.Leaf
        self.Branch = tree_sitter_type_provider.Branch
        self.Point = tree_sitter_type_provider.Point
        self.ParseError = tree_sitter_type_provider.ParseError

    def parse(
        self,
        contents: typing.Union[str, bytes],
        *,
        encoding: str = "utf-8",
        filename: typing.Optional[str] = None,
        raise_parse_error: bool = False,
    ) -> typing.Union[
        typing.Sequence[tree_sitter_type_provider.Node],
        tree_sitter_type_provider.Node,
        None,
    ]:
        tree = self._parse(contents, encoding=encoding)
        return self.from_tree_sitter(
            tree.root_node, filename=filename, raise_parse_error=raise_parse_error
        )

    def parse_file(
        self,
        path: typing.Union[str, pathlib.Path],
        *,
        encoding: str = "utf-8",
        raise_parse_error: bool = False,
    ) -> typing.Union[
        typing.Sequence[tree_sitter_type_provider.Node],
        tree_sitter_type_provider.Node,
        None,
    ]:
        tree = self._parse_file(path, encoding=encoding)
        return self.from_tree_sitter(
            tree.root_node, filename=str(path), raise_parse_error=raise_parse_error
        )

    def _parse(
        self,
        contents: typing.Union[str, bytes],
        *,
        encoding: str = "utf-8",
    ) -> tree_sitter.Tree:
        if isinstance(contents, str):
            contents = bytes(contents, encoding)
        return self.parser.parse(contents)

    def _parse_file(
        self,
        path: typing.Union[str, pathlib.Path],
        *,
        encoding: str = "utf-8",
    ) -> tree_sitter.Tree:
        if not isinstance(path, pathlib.Path):
            path = pathlib.Path(path)
        contents = path.read_bytes()
        return self._parse(contents, encoding=encoding)



from .dynamic import Branch as Branch
from .dynamic import Leaf as Leaf
from .dynamic import Node as Node
from .dynamic import NodeTypeError as NodeTypeError
from .dynamic import NodeTypeName as NodeTypeName
from .dynamic import ParseError as ParseError
from .dynamic import Point as Point
from .dynamic import TalonAction as TalonAction
from .dynamic import TalonArgumentList as TalonArgumentList
from .dynamic import TalonAssignmentStatement as TalonAssignmentStatement
from .dynamic import TalonBinaryOperator as TalonBinaryOperator
from .dynamic import TalonBlock as TalonBlock
from .dynamic import TalonCapture as TalonCapture
from .dynamic import TalonChoice as TalonChoice
from .dynamic import TalonCommandDeclaration as TalonCommandDeclaration
from .dynamic import TalonComment as TalonComment
from .dynamic import TalonDeclaration as TalonDeclaration
from .dynamic import TalonEndAnchor as TalonEndAnchor
from .dynamic import TalonError as TalonError
from .dynamic import TalonExpression as TalonExpression
from .dynamic import TalonExpressionStatement as TalonExpressionStatement
from .dynamic import TalonFloat as TalonFloat
from .dynamic import TalonIdentifier as TalonIdentifier
from .dynamic import TalonImplicitString as TalonImplicitString
from .dynamic import TalonInteger as TalonInteger
from .dynamic import TalonInterpolation as TalonInterpolation
from .dynamic import TalonKeyAction as TalonKeyAction
from .dynamic import TalonKeyBindingDeclaration as TalonKeyBindingDeclaration
from .dynamic import TalonList as TalonList
from .dynamic import TalonMatch as TalonMatch
from .dynamic import TalonMatches as TalonMatches
from .dynamic import TalonMatchModifier as TalonMatchModifier
from .dynamic import TalonNumber as TalonNumber
from .dynamic import TalonOperator as TalonOperator
from .dynamic import TalonOptional as TalonOptional
from .dynamic import TalonParenthesizedExpression as TalonParenthesizedExpression
from .dynamic import TalonParenthesizedRule as TalonParenthesizedRule
from .dynamic import TalonRepeat as TalonRepeat
from .dynamic import TalonRepeat1 as TalonRepeat1
from .dynamic import TalonRule as TalonRule
from .dynamic import TalonSeq as TalonSeq
from .dynamic import TalonSettingsDeclaration as TalonSettingsDeclaration
from .dynamic import TalonSleepAction as TalonSleepAction
from .dynamic import TalonSourceFile as TalonSourceFile
from .dynamic import TalonStartAnchor as TalonStartAnchor
from .dynamic import TalonStatement as TalonStatement
from .dynamic import TalonString as TalonString
from .dynamic import TalonStringContent as TalonStringContent
from .dynamic import TalonStringEscapeSequence as TalonStringEscapeSequence
from .dynamic import TalonTagImportDeclaration as TalonTagImportDeclaration
from .dynamic import TalonVariable as TalonVariable
from .dynamic import TalonWord as TalonWord
from .dynamic import __grammar_version__ as __grammar_version__
from .dynamic import __version__ as __version__
from .dynamic import from_tree_sitter as from_tree_sitter
from .dynamic import language as language
from .dynamic import parse as parse
from .dynamic import parse_file as parse_file
from .dynamic import parser as parser
from .extra import __pickle_compat



import pytest

import tree_sitter_talon

from . import node_dict_simplify


@pytest.mark.golden_test("data/golden/*.yml")
def test_golden(golden):
    node_dict = tree_sitter_talon.parse(golden["input"]).to_dict()
    node_dict_simplify(node_dict)
    assert node_dict == golden.out["output"]



import dataclasses
import pathlib
import typing

import tree_sitter  # type: ignore
from tree_sitter_type_provider import Branch as Branch
from tree_sitter_type_provider import Leaf as Leaf
from tree_sitter_type_provider import Node as Node
from tree_sitter_type_provider import NodeTypeError as NodeTypeError
from tree_sitter_type_provider import NodeTypeName as NodeTypeName
from tree_sitter_type_provider import ParseError as ParseError
from tree_sitter_type_provider import Point as Point

__version__: str

__grammar_version__: str

parser: tree_sitter.Parser

language: tree_sitter.Language

def parse(
    contents: typing.Union[str, bytes],
    *,
    encoding: str = "utf-8",
    filename: typing.Optional[str] = None,
    raise_parse_error: bool = False,
) -> Node: ...
def parse_file(
    path: typing.Union[str, pathlib.Path],
    *,
    encoding: str = "utf-8",
    raise_parse_error: bool = False,
) -> Node: ...
def from_tree_sitter(
    tsvalue: typing.Union[tree_sitter.Tree, tree_sitter.Node, tree_sitter.TreeCursor],
    *,
    encoding: str = "utf-8",
    filename: typing.Optional[str] = None,
    raise_parse_error: bool = False,
) -> Node: ...

# AST node classes.

@dataclasses.dataclass
class TalonSourceFile(Branch):
    children: typing.Sequence[
        typing.Union[TalonDeclaration, TalonMatches, TalonComment]
    ]

class TalonComment(Leaf):
    pass

@dataclasses.dataclass
class TalonError(Exception, Branch):
    children: typing.Sequence[Node]
    contents: typing.Optional[str] = None
    filename: typing.Optional[str] = None

# Matches.

@dataclasses.dataclass
class TalonMatches(Branch):
    children: typing.Sequence[typing.Union[TalonMatch, TalonComment]]

@dataclasses.dataclass
class TalonMatch(Branch):
    children: typing.Sequence[TalonComment]
    key: TalonIdentifier
    modifier: typing.Sequence[TalonMatchModifier]
    pattern: TalonImplicitString

class TalonMatchModifier(Leaf):
    pass

# Declarations.

class TalonDeclaration(Node):
    pass

@dataclasses.dataclass
class TalonCommandDeclaration(Branch, TalonDeclaration):
    children: typing.Sequence[TalonComment]
    rule: TalonRule
    script: TalonBlock

@dataclasses.dataclass
class TalonKeyBindingDeclaration(Branch, TalonDeclaration):
    children: typing.Sequence[TalonComment]
    key: TalonKeyAction
    script: TalonBlock

@dataclasses.dataclass
class TalonSettingsDeclaration(Branch, TalonDeclaration):
    children: typing.Sequence[typing.Union[TalonBlock, TalonComment]]

@dataclasses.dataclass
class TalonTagImportDeclaration(Branch, TalonDeclaration):
    children: typing.Sequence[TalonComment]
    tag: TalonIdentifier

# Rules.

@dataclasses.dataclass
class TalonCapture(Branch):
    children: typing.Sequence[TalonComment]
    capture_name: TalonIdentifier

@dataclasses.dataclass
class TalonChoice(Branch):
    children: typing.Sequence[
        typing.Union[
            TalonCapture,
            TalonEndAnchor,
            TalonList,
            TalonOptional,
            TalonParenthesizedRule,
            TalonRepeat,
            TalonRepeat1,
            TalonSeq,
            TalonStartAnchor,
            TalonWord,
            TalonComment,
        ]
    ]

class TalonEndAnchor(Leaf):
    pass

@dataclasses.dataclass
class TalonList(Branch):
    children: typing.Sequence[TalonComment]
    list_name: TalonIdentifier

@dataclasses.dataclass
class TalonOptional(Branch):
    children: typing.Sequence[
        typing.Union[
            TalonCapture,
            TalonChoice,
            TalonEndAnchor,
            TalonList,
            TalonOptional,
            TalonParenthesizedRule,
            TalonRepeat,
            TalonRepeat1,
            TalonSeq,
            TalonStartAnchor,
            TalonWord,
            TalonComment,
        ]
    ]

@dataclasses.dataclass
class TalonParenthesizedRule(Branch):
    children: typing.Sequence[
        typing.Union[
            TalonCapture,
            TalonChoice,
            TalonEndAnchor,
            TalonList,
            TalonOptional,
            TalonParenthesizedRule,
            TalonRepeat,
            TalonRepeat1,
            TalonSeq,
            TalonStartAnchor,
            TalonWord,
            TalonComment,
        ]
    ]

@dataclasses.dataclass
class TalonRepeat(Branch):
    children: typing.Sequence[
        typing.Union[
            TalonCapture,
            TalonList,
            TalonOptional,
            TalonParenthesizedRule,
            TalonRepeat,
            TalonRepeat1,
            TalonWord,
            TalonComment,
        ]
    ]

@dataclasses.dataclass
class TalonRepeat1(Branch):
    children: typing.Sequence[
        typing.Union[
            TalonCapture,
            TalonList,
            TalonOptional,
            TalonParenthesizedRule,
            TalonRepeat,
            TalonRepeat1,
            TalonWord,
            TalonComment,
        ]
    ]

@dataclasses.dataclass
class TalonRule(Branch):
    children: typing.Sequence[
        typing.Union[
            TalonCapture,
            TalonChoice,
            TalonEndAnchor,
            TalonList,
            TalonOptional,
            TalonParenthesizedRule,
            TalonRepeat,
            TalonRepeat1,
            TalonSeq,
            TalonStartAnchor,
            TalonWord,
            TalonComment,
        ]
    ]

@dataclasses.dataclass
class TalonSeq(Branch):
    children: typing.Sequence[
        typing.Union[
            TalonCapture,
            TalonList,
            TalonOptional,
            TalonParenthesizedRule,
            TalonRepeat,
            TalonRepeat1,
            TalonWord,
            TalonComment,
        ]
    ]

class TalonStartAnchor(Leaf):
    pass

class TalonWord(Leaf):
    pass

# Statements.

class TalonStatement(Node):
    pass

@dataclasses.dataclass
class TalonAssignmentStatement(Branch, TalonStatement):
    children: typing.Sequence[TalonComment]
    left: TalonIdentifier
    right: TalonExpression

@dataclasses.dataclass
class TalonExpressionStatement(Branch, TalonStatement):
    children: typing.Sequence[TalonComment]
    expression: TalonExpression

@dataclasses.dataclass
class TalonBlock(Branch):
    children: typing.Sequence[typing.Union[TalonStatement, TalonComment]]

# Expressions.

class TalonExpression(Node):
    pass

@dataclasses.dataclass
class TalonAction(Branch, TalonExpression):
    children: typing.Sequence[TalonComment]
    action_name: TalonIdentifier
    arguments: TalonArgumentList

@dataclasses.dataclass
class TalonArgumentList(Branch):
    children: typing.Sequence[typing.Union[TalonExpression, TalonComment]]

@dataclasses.dataclass
class TalonBinaryOperator(Branch, TalonExpression):
    children: typing.Sequence[TalonComment]
    left: TalonExpression
    operator: TalonOperator
    right: TalonExpression

@dataclasses.dataclass
class TalonKeyAction(Branch, TalonExpression):
    children: typing.Sequence[TalonComment]
    arguments: TalonImplicitString

@dataclasses.dataclass
class TalonParenthesizedExpression(Branch, TalonExpression):
    children: typing.Sequence[typing.Union[TalonExpression, TalonComment]]

@dataclasses.dataclass
class TalonSleepAction(Branch, TalonExpression):
    children: typing.Sequence[TalonComment]
    arguments: TalonImplicitString

@dataclasses.dataclass
class TalonVariable(Branch, TalonExpression):
    children: typing.Sequence[TalonComment]
    variable_name: TalonIdentifier

# Identifiers.

class TalonIdentifier(Leaf):
    pass

class TalonOperator(Leaf):
    pass

# Strings.

class TalonImplicitString(Leaf):
    pass

@dataclasses.dataclass
class TalonInterpolation(Branch):
    children: typing.Sequence[typing.Union[TalonExpression, TalonComment]]

@dataclasses.dataclass
class TalonString(Branch):
    children: typing.Sequence[
        typing.Union[
            TalonInterpolation,
            TalonStringContent,
            TalonStringEscapeSequence,
            TalonComment,
        ]
    ]

class TalonStringContent(Leaf):
    pass

class TalonStringEscapeSequence(Leaf):
    pass

# Numbers.

class TalonFloat(Leaf, TalonNumber):
    pass

class TalonInteger(Leaf, TalonNumber):
    pass

class TalonNumber(Node):
    pass



import setuptools  # isort:skip
import distutils.extension
import os

with open(os.path.join(os.path.dirname(__file__), "README.md")) as f:
    LONG_DESCRIPTION = f.read()

setuptools.setup(
    name="tree_sitter_talon",
    version="3.0.5",
    description="Parser for Talon files in Python",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    author="Wen Kokke",
    url="https://github.com/wenkokke/py-tree-sitter-talon",
    project_urls={
        "Documentation": "https://wenkokke.github.io/py-tree-sitter-talon/",
        "Source": "https://github.com/tree-sitter/py-tree-sitter",
    },
    packages=setuptools.find_packages(),
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Compilers",
    ],
    license="MIT",
    keywords=["talon", "parser"],
    platforms=["any"],
    python_requires=">=3.9",
    setup_requires=[
        "pytest-runner",
    ],
    tests_require=[
        "pytest >=7.1.2, <8",
        "pytest-golden >=0.2.2, <0.3",
    ],
    install_requires=[
        "tree_sitter ==0.20.0",
        "tree_sitter_type_provider ==2.1.6",
        "dataclasses-json >=0.5.7, <0.6",
    ],
    extras_require={
        "dev": [
            "build >=0.8.0, <0.9",
            "mypy >=0.971, <1",
            "twine >=4.0.1, <5",
        ],
        "doc": [
            "Sphinx >=5.1.1, <6",
            "sphinx-bootstrap-theme >=0.8.1, <0.9",
        ],
        "test": ["pytest >=7.1.2, <8", "pytest-golden >=0.2.2, <0.3"],
    },
    ext_modules=[
        distutils.extension.Extension(
            name="tree_sitter_talon.binding",
            sources=[
                "tree_sitter_talon/binding.c",
                "tree_sitter_talon/data/tree-sitter-talon/src/parser.c",
                "tree_sitter_talon/data/tree-sitter-talon/src/scanner.cc",
            ],
            include_dirs=[
                "tree_sitter_talon/data/tree-sitter-talon/src",
            ],
        )
    ],
    package_data={
        "tree_sitter_talon": [
            "py.typed",
            "binding.c",
            "binding.pyi",
            "dynamic.pyi",
            "data/tree-sitter-talon/binding.gyp",
            "data/tree-sitter-talon/bindings/node/binding.cc",
            "data/tree-sitter-talon/bindings/node/index.js",
            "data/tree-sitter-talon/bindings/rust/build.rs",
            "data/tree-sitter-talon/bindings/rust/lib.rs",
            "data/tree-sitter-talon/grammar.js",
            "data/tree-sitter-talon/package.json",
            "data/tree-sitter-talon/src/grammar.json",
            "data/tree-sitter-talon/src/node-types.json",
            "data/tree-sitter-talon/src/parser.c",
            "data/tree-sitter-talon/src/scanner.cc",
            "data/tree-sitter-talon/src/tree_sitter/parser.h",
        ]
    },
)



import dataclasses

from .. import dynamic as _dynamic


def _make_TalonAction(*args, **kwargs) -> _dynamic.TalonAction:
    return _dynamic.TalonAction(*args, **kwargs)


setattr(
    _dynamic.TalonAction,
    "__reduce__",
    lambda self: (_make_TalonAction, dataclasses.astuple(self)),
)


def _make_TalonArgumentList(*args, **kwargs) -> _dynamic.TalonArgumentList:
    return _dynamic.TalonArgumentList(*args, **kwargs)


setattr(
    _dynamic.TalonArgumentList,
    "__reduce__",
    lambda self: (_make_TalonArgumentList, dataclasses.astuple(self)),
)


def _make_TalonAssignmentStatement(
    *args, **kwargs
) -> _dynamic.TalonAssignmentStatement:
    return _dynamic.TalonAssignmentStatement(*args, **kwargs)


setattr(
    _dynamic.TalonAssignmentStatement,
    "__reduce__",
    lambda self: (_make_TalonAssignmentStatement, dataclasses.astuple(self)),
)


def _make_TalonBinaryOperator(*args, **kwargs) -> _dynamic.TalonBinaryOperator:
    return _dynamic.TalonBinaryOperator(*args, **kwargs)


setattr(
    _dynamic.TalonBinaryOperator,
    "__reduce__",
    lambda self: (_make_TalonBinaryOperator, dataclasses.astuple(self)),
)


def _make_TalonBlock(*args, **kwargs) -> _dynamic.TalonBlock:
    return _dynamic.TalonBlock(*args, **kwargs)


setattr(
    _dynamic.TalonBlock,
    "__reduce__",
    lambda self: (_make_TalonBlock, dataclasses.astuple(self)),
)


def _make_TalonCapture(*args, **kwargs) -> _dynamic.TalonCapture:
    return _dynamic.TalonCapture(*args, **kwargs)


setattr(
    _dynamic.TalonCapture,
    "__reduce__",
    lambda self: (_make_TalonCapture, dataclasses.astuple(self)),
)


def _make_TalonChoice(*args, **kwargs) -> _dynamic.TalonChoice:
    return _dynamic.TalonChoice(*args, **kwargs)


setattr(
    _dynamic.TalonChoice,
    "__reduce__",
    lambda self: (_make_TalonChoice, dataclasses.astuple(self)),
)


def _make_TalonCommandDeclaration(*args, **kwargs) -> _dynamic.TalonCommandDeclaration:
    return _dynamic.TalonCommandDeclaration(*args, **kwargs)


setattr(
    _dynamic.TalonCommandDeclaration,
    "__reduce__",
    lambda self: (_make_TalonCommandDeclaration, dataclasses.astuple(self)),
)


def _make_TalonComment(*args, **kwargs) -> _dynamic.TalonComment:
    return _dynamic.TalonComment(*args, **kwargs)


setattr(
    _dynamic.TalonComment,
    "__reduce__",
    lambda self: (_make_TalonComment, dataclasses.astuple(self)),
)


def _make_TalonDeclaration(*args, **kwargs) -> _dynamic.TalonDeclaration:
    return _dynamic.TalonDeclaration(*args, **kwargs)


setattr(
    _dynamic.TalonDeclaration,
    "__reduce__",
    lambda self: (_make_TalonDeclaration, dataclasses.astuple(self)),
)


def _make_TalonEndAnchor(*args, **kwargs) -> _dynamic.TalonEndAnchor:
    return _dynamic.TalonEndAnchor(*args, **kwargs)


setattr(
    _dynamic.TalonEndAnchor,
    "__reduce__",
    lambda self: (_make_TalonEndAnchor, dataclasses.astuple(self)),
)


def _make_TalonError(*args, **kwargs) -> _dynamic.TalonError:
    return _dynamic.TalonError(*args, **kwargs)


setattr(
    _dynamic.TalonError,
    "__reduce__",
    lambda self: (_make_TalonError, dataclasses.astuple(self)),
)


def _make_TalonExpression(*args, **kwargs) -> _dynamic.TalonExpression:
    return _dynamic.TalonExpression(*args, **kwargs)


setattr(
    _dynamic.TalonExpression,
    "__reduce__",
    lambda self: (_make_TalonExpression, dataclasses.astuple(self)),
)


def _make_TalonExpressionStatement(
    *args, **kwargs
) -> _dynamic.TalonExpressionStatement:
    return _dynamic.TalonExpressionStatement(*args, **kwargs)


setattr(
    _dynamic.TalonExpressionStatement,
    "__reduce__",
    lambda self: (_make_TalonExpressionStatement, dataclasses.astuple(self)),
)


def _make_TalonFloat(*args, **kwargs) -> _dynamic.TalonFloat:
    return _dynamic.TalonFloat(*args, **kwargs)


setattr(
    _dynamic.TalonFloat,
    "__reduce__",
    lambda self: (_make_TalonFloat, dataclasses.astuple(self)),
)


def _make_TalonIdentifier(*args, **kwargs) -> _dynamic.TalonIdentifier:
    return _dynamic.TalonIdentifier(*args, **kwargs)


setattr(
    _dynamic.TalonIdentifier,
    "__reduce__",
    lambda self: (_make_TalonIdentifier, dataclasses.astuple(self)),
)


def _make_TalonImplicitString(*args, **kwargs) -> _dynamic.TalonImplicitString:
    return _dynamic.TalonImplicitString(*args, **kwargs)


setattr(
    _dynamic.TalonImplicitString,
    "__reduce__",
    lambda self: (_make_TalonImplicitString, dataclasses.astuple(self)),
)


def _make_TalonInteger(*args, **kwargs) -> _dynamic.TalonInteger:
    return _dynamic.TalonInteger(*args, **kwargs)


setattr(
    _dynamic.TalonInteger,
    "__reduce__",
    lambda self: (_make_TalonInteger, dataclasses.astuple(self)),
)


def _make_TalonInterpolation(*args, **kwargs) -> _dynamic.TalonInterpolation:
    return _dynamic.TalonInterpolation(*args, **kwargs)


setattr(
    _dynamic.TalonInterpolation,
    "__reduce__",
    lambda self: (_make_TalonInterpolation, dataclasses.astuple(self)),
)


def _make_TalonKeyAction(*args, **kwargs) -> _dynamic.TalonKeyAction:
    return _dynamic.TalonKeyAction(*args, **kwargs)


setattr(
    _dynamic.TalonKeyAction,
    "__reduce__",
    lambda self: (_make_TalonKeyAction, dataclasses.astuple(self)),
)


def _make_TalonKeyBindingDeclaration(
    *args, **kwargs
) -> _dynamic.TalonKeyBindingDeclaration:
    return _dynamic.TalonKeyBindingDeclaration(*args, **kwargs)


setattr(
    _dynamic.TalonKeyBindingDeclaration,
    "__reduce__",
    lambda self: (_make_TalonKeyBindingDeclaration, dataclasses.astuple(self)),
)


def _make_TalonList(*args, **kwargs) -> _dynamic.TalonList:
    return _dynamic.TalonList(*args, **kwargs)


setattr(
    _dynamic.TalonList,
    "__reduce__",
    lambda self: (_make_TalonList, dataclasses.astuple(self)),
)


def _make_TalonMatch(*args, **kwargs) -> _dynamic.TalonMatch:
    return _dynamic.TalonMatch(*args, **kwargs)


setattr(
    _dynamic.TalonMatch,
    "__reduce__",
    lambda self: (_make_TalonMatch, dataclasses.astuple(self)),
)


def _make_TalonMatches(*args, **kwargs) -> _dynamic.TalonMatches:
    return _dynamic.TalonMatches(*args, **kwargs)


setattr(
    _dynamic.TalonMatches,
    "__reduce__",
    lambda self: (_make_TalonMatches, dataclasses.astuple(self)),
)


def _make_TalonMatchModifier(*args, **kwargs) -> _dynamic.TalonMatchModifier:
    return _dynamic.TalonMatchModifier(*args, **kwargs)


setattr(
    _dynamic.TalonMatchModifier,
    "__reduce__",
    lambda self: (_make_TalonMatchModifier, dataclasses.astuple(self)),
)


def _make_TalonNumber(*args, **kwargs) -> _dynamic.TalonNumber:
    return _dynamic.TalonNumber(*args, **kwargs)


setattr(
    _dynamic.TalonNumber,
    "__reduce__",
    lambda self: (_make_TalonNumber, dataclasses.astuple(self)),
)


def _make_TalonOperator(*args, **kwargs) -> _dynamic.TalonOperator:
    return _dynamic.TalonOperator(*args, **kwargs)


setattr(
    _dynamic.TalonOperator,
    "__reduce__",
    lambda self: (_make_TalonOperator, dataclasses.astuple(self)),
)


def _make_TalonOptional(*args, **kwargs) -> _dynamic.TalonOptional:
    return _dynamic.TalonOptional(*args, **kwargs)


setattr(
    _dynamic.TalonOptional,
    "__reduce__",
    lambda self: (_make_TalonOptional, dataclasses.astuple(self)),
)


def _make_TalonParenthesizedExpression(
    *args, **kwargs
) -> _dynamic.TalonParenthesizedExpression:
    return _dynamic.TalonParenthesizedExpression(*args, **kwargs)


setattr(
    _dynamic.TalonParenthesizedExpression,
    "__reduce__",
    lambda self: (_make_TalonParenthesizedExpression, dataclasses.astuple(self)),
)


def _make_TalonParenthesizedRule(*args, **kwargs) -> _dynamic.TalonParenthesizedRule:
    return _dynamic.TalonParenthesizedRule(*args, **kwargs)


setattr(
    _dynamic.TalonParenthesizedRule,
    "__reduce__",
    lambda self: (_make_TalonParenthesizedRule, dataclasses.astuple(self)),
)


def _make_TalonRepeat(*args, **kwargs) -> _dynamic.TalonRepeat:
    return _dynamic.TalonRepeat(*args, **kwargs)


setattr(
    _dynamic.TalonRepeat,
    "__reduce__",
    lambda self: (_make_TalonRepeat, dataclasses.astuple(self)),
)


def _make_TalonRepeat1(*args, **kwargs) -> _dynamic.TalonRepeat1:
    return _dynamic.TalonRepeat1(*args, **kwargs)


setattr(
    _dynamic.TalonRepeat1,
    "__reduce__",
    lambda self: (_make_TalonRepeat1, dataclasses.astuple(self)),
)


def _make_TalonRule(*args, **kwargs) -> _dynamic.TalonRule:
    return _dynamic.TalonRule(*args, **kwargs)


setattr(
    _dynamic.TalonRule,
    "__reduce__",
    lambda self: (_make_TalonRule, dataclasses.astuple(self)),
)


def _make_TalonSeq(*args, **kwargs) -> _dynamic.TalonSeq:
    return _dynamic.TalonSeq(*args, **kwargs)


setattr(
    _dynamic.TalonSeq,
    "__reduce__",
    lambda self: (_make_TalonSeq, dataclasses.astuple(self)),
)


def _make_TalonSettingsDeclaration(
    *args, **kwargs
) -> _dynamic.TalonSettingsDeclaration:
    return _dynamic.TalonSettingsDeclaration(*args, **kwargs)


setattr(
    _dynamic.TalonSettingsDeclaration,
    "__reduce__",
    lambda self: (_make_TalonSettingsDeclaration, dataclasses.astuple(self)),
)


def _make_TalonSleepAction(*args, **kwargs) -> _dynamic.TalonSleepAction:
    return _dynamic.TalonSleepAction(*args, **kwargs)


setattr(
    _dynamic.TalonSleepAction,
    "__reduce__",
    lambda self: (_make_TalonSleepAction, dataclasses.astuple(self)),
)


def _make_TalonSourceFile(*args, **kwargs) -> _dynamic.TalonSourceFile:
    return _dynamic.TalonSourceFile(*args, **kwargs)


setattr(
    _dynamic.TalonSourceFile,
    "__reduce__",
    lambda self: (_make_TalonSourceFile, dataclasses.astuple(self)),
)


def _make_TalonStartAnchor(*args, **kwargs) -> _dynamic.TalonStartAnchor:
    return _dynamic.TalonStartAnchor(*args, **kwargs)


setattr(
    _dynamic.TalonStartAnchor,
    "__reduce__",
    lambda self: (_make_TalonStartAnchor, dataclasses.astuple(self)),
)


def _make_TalonStatement(*args, **kwargs) -> _dynamic.TalonStatement:
    return _dynamic.TalonStatement(*args, **kwargs)


setattr(
    _dynamic.TalonStatement,
    "__reduce__",
    lambda self: (_make_TalonStatement, dataclasses.astuple(self)),
)


def _make_TalonString(*args, **kwargs) -> _dynamic.TalonString:
    return _dynamic.TalonString(*args, **kwargs)


setattr(
    _dynamic.TalonString,
    "__reduce__",
    lambda self: (_make_TalonString, dataclasses.astuple(self)),
)


def _make_TalonStringContent(*args, **kwargs) -> _dynamic.TalonStringContent:
    return _dynamic.TalonStringContent(*args, **kwargs)


setattr(
    _dynamic.TalonStringContent,
    "__reduce__",
    lambda self: (_make_TalonStringContent, dataclasses.astuple(self)),
)


def _make_TalonStringEscapeSequence(
    *args, **kwargs
) -> _dynamic.TalonStringEscapeSequence:
    return _dynamic.TalonStringEscapeSequence(*args, **kwargs)


setattr(
    _dynamic.TalonStringEscapeSequence,
    "__reduce__",
    lambda self: (_make_TalonStringEscapeSequence, dataclasses.astuple(self)),
)


def _make_TalonTagImportDeclaration(
    *args, **kwargs
) -> _dynamic.TalonTagImportDeclaration:
    return _dynamic.TalonTagImportDeclaration(*args, **kwargs)


setattr(
    _dynamic.TalonTagImportDeclaration,
    "__reduce__",
    lambda self: (_make_TalonTagImportDeclaration, dataclasses.astuple(self)),
)


def _make_TalonVariable(*args, **kwargs) -> _dynamic.TalonVariable:
    return _dynamic.TalonVariable(*args, **kwargs)


setattr(
    _dynamic.TalonVariable,
    "__reduce__",
    lambda self: (_make_TalonVariable, dataclasses.astuple(self)),
)


def _make_TalonWord(*args, **kwargs) -> _dynamic.TalonWord:
    return _dynamic.TalonWord(*args, **kwargs)


setattr(
    _dynamic.TalonWord,
    "__reduce__",
    lambda self: (_make_TalonWord, dataclasses.astuple(self)),
)
